#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2012-10-20 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ../scripts/aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
@todo: FIX: put in a description of the message here with fields and types.
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = (
	'MessageID',
	'RepeatIndicator',
	'UserID',
	'Reserved1',
	'SOG',
	'PositionAccuracy',
	'longitude',
	'latitude',
	'COG',
	'TrueHeading',
	'TimeStamp',
	'Spare',
	'cs_unit',
	'display_flag',
	'dsc_flag',
	'band_flag',
	'msg22_flag',
	'mode_flag',
	'RAIM',
	'CommStateSelector',
	'CommState',
)

fieldListPostgres = (
	'MessageID',
	'RepeatIndicator',
	'UserID',
	'Reserved1',
	'SOG',
	'PositionAccuracy',
	'Position',	# PostGIS data type
	'COG',
	'TrueHeading',
	'TimeStamp',
	'Spare',
	'cs_unit',
	'display_flag',
	'dsc_flag',
	'band_flag',
	'msg22_flag',
	'mode_flag',
	'RAIM',
	'CommStateSelector',
	'CommState',
)

toPgFields = {
	'longitude':'Position',
	'latitude':'Position',
}
'''
Go to the Postgis field names from the straight field name
'''

fromPgFields = {
	'Position':('longitude','latitude',),
}
'''
Go from the Postgis field names to the straight field name
'''

pgTypes = {
	'Position':'POINT',
}
'''
Lookup table for each postgis field name to get its type.
'''

def encode(params, validate=False):
	'''Create a positionb binary message payload to pack into an AIS Msg positionb.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 18 (field automatically set to "18")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - Reserved1(uint): Reseverd for definition by a compentent regional or local authority.  Should be set to zero. (field automatically set to "0")
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - longitude(decimal): Location of the vessel  East West location
	  - latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - Spare(uint): Not used.  Should be set to zero. (field automatically set to "0")
	  - cs_unit(bool): Does this unit do Carrier Sense?
	  - display_flag(bool): Does this class B unit have an integrated display?
	  - dsc_flag(bool): Does it have dedicated or time-shared DSC radio function?
	  - band_flag(bool): How flexible is the freq handling of the unit?
	  - msg22_flag(bool): Can the unit handle msg 22?
	  - mode_flag(bool): Assigned mode wrt to VDL slots
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - CommStateSelector(uint): SOTDMA or ITDMA
	  - CommState(uint): Not decoded by this software yet
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=18),6))
	if 'RepeatIndicator' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['RepeatIndicator']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['UserID']),30))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),8))
	if 'SOG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['SOG'])*Decimal('10')))),10))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(1023)),10))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['PositionAccuracy']),1))
	if 'longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'COG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['COG'])*Decimal('10')))),12))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(3600)),12))
	if 'TrueHeading' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TrueHeading']),9))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=511),9))
	if 'TimeStamp' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TimeStamp']),6))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=60),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	if params["cs_unit"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["display_flag"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["dsc_flag"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["band_flag"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["msg22_flag"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["mode_flag"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["RAIM"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['CommStateSelector']),1))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['CommState']),19))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a positionb message 

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 18 (field automatically set to "18")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - Reserved1(uint): Reseverd for definition by a compentent regional or local authority.  Should be set to zero. (field automatically set to "0")
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - longitude(decimal): Location of the vessel  East West location
	  - latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - Spare(uint): Not used.  Should be set to zero. (field automatically set to "0")
	  - cs_unit(bool): Does this unit do Carrier Sense?
	  - display_flag(bool): Does this class B unit have an integrated display?
	  - dsc_flag(bool): Does it have dedicated or time-shared DSC radio function?
	  - band_flag(bool): How flexible is the freq handling of the unit?
	  - msg22_flag(bool): Can the unit handle msg 22?
	  - mode_flag(bool): Assigned mode wrt to VDL slots
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - CommStateSelector(uint): SOTDMA or ITDMA
	  - CommState(uint): Not decoded by this software yet
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['MessageID']=18
	r['RepeatIndicator']=int(bv[6:8])
	r['UserID']=int(bv[8:38])
	r['Reserved1']=0
	r['SOG']=Decimal(int(bv[46:56]))/Decimal('10')
	r['PositionAccuracy']=int(bv[56:57])
	r['longitude']=Decimal(binary.signedIntFromBV(bv[57:85]))/Decimal('600000')
	r['latitude']=Decimal(binary.signedIntFromBV(bv[85:112]))/Decimal('600000')
	r['COG']=Decimal(int(bv[112:124]))/Decimal('10')
	r['TrueHeading']=int(bv[124:133])
	r['TimeStamp']=int(bv[133:139])
	r['Spare']=0
	r['cs_unit']=bool(int(bv[141:142]))
	r['display_flag']=bool(int(bv[142:143]))
	r['dsc_flag']=bool(int(bv[143:144]))
	r['band_flag']=bool(int(bv[144:145]))
	r['msg22_flag']=bool(int(bv[145:146]))
	r['mode_flag']=bool(int(bv[146:147]))
	r['RAIM']=bool(int(bv[147:148]))
	r['CommStateSelector']=int(bv[148:149])
	r['CommState']=int(bv[149:168])
	return r

def decodeMessageID(bv, validate=False):
	return 18

def decodeRepeatIndicator(bv, validate=False):
	return int(bv[6:8])

def decodeUserID(bv, validate=False):
	return int(bv[8:38])

def decodeReserved1(bv, validate=False):
	return 0

def decodeSOG(bv, validate=False):
	return Decimal(int(bv[46:56]))/Decimal('10')

def decodePositionAccuracy(bv, validate=False):
	return int(bv[56:57])

def decodelongitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[57:85]))/Decimal('600000')

def decodelatitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[85:112]))/Decimal('600000')

def decodeCOG(bv, validate=False):
	return Decimal(int(bv[112:124]))/Decimal('10')

def decodeTrueHeading(bv, validate=False):
	return int(bv[124:133])

def decodeTimeStamp(bv, validate=False):
	return int(bv[133:139])

def decodeSpare(bv, validate=False):
	return 0

def decodecs_unit(bv, validate=False):
	return bool(int(bv[141:142]))

def decodedisplay_flag(bv, validate=False):
	return bool(int(bv[142:143]))

def decodedsc_flag(bv, validate=False):
	return bool(int(bv[143:144]))

def decodeband_flag(bv, validate=False):
	return bool(int(bv[144:145]))

def decodemsg22_flag(bv, validate=False):
	return bool(int(bv[145:146]))

def decodemode_flag(bv, validate=False):
	return bool(int(bv[146:147]))

def decodeRAIM(bv, validate=False):
	return bool(int(bv[147:148]))

def decodeCommStateSelector(bv, validate=False):
	return int(bv[148:149])

def decodeCommState(bv, validate=False):
	return int(bv[149:168])


def printHtml(params, out=sys.stdout):
		out.write("<h3>positionb</h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>MessageID</td>\n")
		out.write("<td>uint</td>\n")
		if 'MessageID' in params:
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RepeatIndicator</td>\n")
		out.write("<td>uint</td>\n")
		if 'RepeatIndicator' in params:
			out.write("	<td>"+str(params['RepeatIndicator'])+"</td>\n")
			if str(params['RepeatIndicator']) in RepeatIndicatorDecodeLut:
				out.write("<td>"+RepeatIndicatorDecodeLut[str(params['RepeatIndicator'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>UserID</td>\n")
		out.write("<td>uint</td>\n")
		if 'UserID' in params:
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Reserved1</td>\n")
		out.write("<td>uint</td>\n")
		if 'Reserved1' in params:
			out.write("	<td>"+str(params['Reserved1'])+"</td>\n")
			out.write("	<td>"+str(params['Reserved1'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>SOG</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'SOG' in params:
			out.write("	<td>"+str(params['SOG'])+"</td>\n")
			if str(params['SOG']) in SOGDecodeLut:
				out.write("<td>"+SOGDecodeLut[str(params['SOG'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>knots</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>PositionAccuracy</td>\n")
		out.write("<td>uint</td>\n")
		if 'PositionAccuracy' in params:
			out.write("	<td>"+str(params['PositionAccuracy'])+"</td>\n")
			if str(params['PositionAccuracy']) in PositionAccuracyDecodeLut:
				out.write("<td>"+PositionAccuracyDecodeLut[str(params['PositionAccuracy'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'longitude' in params:
			out.write("	<td>"+str(params['longitude'])+"</td>\n")
			out.write("	<td>"+str(params['longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'latitude' in params:
			out.write("	<td>"+str(params['latitude'])+"</td>\n")
			out.write("	<td>"+str(params['latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>COG</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'COG' in params:
			out.write("	<td>"+str(params['COG'])+"</td>\n")
			out.write("	<td>"+str(params['COG'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>TrueHeading</td>\n")
		out.write("<td>uint</td>\n")
		if 'TrueHeading' in params:
			out.write("	<td>"+str(params['TrueHeading'])+"</td>\n")
			out.write("	<td>"+str(params['TrueHeading'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>TimeStamp</td>\n")
		out.write("<td>uint</td>\n")
		if 'TimeStamp' in params:
			out.write("	<td>"+str(params['TimeStamp'])+"</td>\n")
			if str(params['TimeStamp']) in TimeStampDecodeLut:
				out.write("<td>"+TimeStampDecodeLut[str(params['TimeStamp'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>seconds</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Spare</td>\n")
		out.write("<td>uint</td>\n")
		if 'Spare' in params:
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>cs_unit</td>\n")
		out.write("<td>bool</td>\n")
		if 'cs_unit' in params:
			out.write("	<td>"+str(params['cs_unit'])+"</td>\n")
			if str(params['cs_unit']) in cs_unitDecodeLut:
				out.write("<td>"+cs_unitDecodeLut[str(params['cs_unit'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>display_flag</td>\n")
		out.write("<td>bool</td>\n")
		if 'display_flag' in params:
			out.write("	<td>"+str(params['display_flag'])+"</td>\n")
			if str(params['display_flag']) in display_flagDecodeLut:
				out.write("<td>"+display_flagDecodeLut[str(params['display_flag'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dsc_flag</td>\n")
		out.write("<td>bool</td>\n")
		if 'dsc_flag' in params:
			out.write("	<td>"+str(params['dsc_flag'])+"</td>\n")
			if str(params['dsc_flag']) in dsc_flagDecodeLut:
				out.write("<td>"+dsc_flagDecodeLut[str(params['dsc_flag'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>band_flag</td>\n")
		out.write("<td>bool</td>\n")
		if 'band_flag' in params:
			out.write("	<td>"+str(params['band_flag'])+"</td>\n")
			if str(params['band_flag']) in band_flagDecodeLut:
				out.write("<td>"+band_flagDecodeLut[str(params['band_flag'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>msg22_flag</td>\n")
		out.write("<td>bool</td>\n")
		if 'msg22_flag' in params:
			out.write("	<td>"+str(params['msg22_flag'])+"</td>\n")
			if str(params['msg22_flag']) in msg22_flagDecodeLut:
				out.write("<td>"+msg22_flagDecodeLut[str(params['msg22_flag'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>mode_flag</td>\n")
		out.write("<td>bool</td>\n")
		if 'mode_flag' in params:
			out.write("	<td>"+str(params['mode_flag'])+"</td>\n")
			if str(params['mode_flag']) in mode_flagDecodeLut:
				out.write("<td>"+mode_flagDecodeLut[str(params['mode_flag'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RAIM</td>\n")
		out.write("<td>bool</td>\n")
		if 'RAIM' in params:
			out.write("	<td>"+str(params['RAIM'])+"</td>\n")
			if str(params['RAIM']) in RAIMDecodeLut:
				out.write("<td>"+RAIMDecodeLut[str(params['RAIM'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>CommStateSelector</td>\n")
		out.write("<td>uint</td>\n")
		if 'CommStateSelector' in params:
			out.write("	<td>"+str(params['CommStateSelector'])+"</td>\n")
			if str(params['CommStateSelector']) in CommStateSelectorDecodeLut:
				out.write("<td>"+CommStateSelectorDecodeLut[str(params['CommStateSelector'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>CommState</td>\n")
		out.write("<td>uint</td>\n")
		if 'CommState' in params:
			out.write("	<td>"+str(params['CommState'])+"</td>\n")
			out.write("	<td>"+str(params['CommState'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")


def printKml(params, out=sys.stdout):
	'''KML (Keyhole Markup Language) for Google Earth, but without the header/footer'''
	out.write("\	<Placemark>\n")
	out.write("\t	<name>"+str(params['UserID'])+"</name>\n")
	out.write("\t\t<description>\n")
	import StringIO
	buf = StringIO.StringIO()
	printHtml(params,buf)
	import cgi
	out.write(cgi.escape(buf.getvalue()))
	out.write("\t\t</description>\n")
	out.write("\t\t<styleUrl>#m_ylw-pushpin_copy0</styleUrl>\n")
	out.write("\t\t<Point>\n")
	out.write("\t\t\t<coordinates>")
	out.write(str(params['longitude']))
	out.write(',')
	out.write(str(params['latitude']))
	out.write(",0</coordinates>\n")
	out.write("\t\t</Point>\n")
	out.write("\t</Placemark>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None, dbType='postgres'):
	'''Print a positionb message to stdout.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 18 (field automatically set to "18")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - Reserved1(uint): Reseverd for definition by a compentent regional or local authority.  Should be set to zero. (field automatically set to "0")
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - longitude(decimal): Location of the vessel  East West location
	  - latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - Spare(uint): Not used.  Should be set to zero. (field automatically set to "0")
	  - cs_unit(bool): Does this unit do Carrier Sense?
	  - display_flag(bool): Does this class B unit have an integrated display?
	  - dsc_flag(bool): Does it have dedicated or time-shared DSC radio function?
	  - band_flag(bool): How flexible is the freq handling of the unit?
	  - msg22_flag(bool): Can the unit handle msg 22?
	  - mode_flag(bool): Assigned mode wrt to VDL slots
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - CommStateSelector(uint): SOTDMA or ITDMA
	  - CommState(uint): Not decoded by this software yet
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("positionb:\n")
		if 'MessageID' in params: out.write("	MessageID:          "+str(params['MessageID'])+"\n")
		if 'RepeatIndicator' in params: out.write("	RepeatIndicator:    "+str(params['RepeatIndicator'])+"\n")
		if 'UserID' in params: out.write("	UserID:             "+str(params['UserID'])+"\n")
		if 'Reserved1' in params: out.write("	Reserved1:          "+str(params['Reserved1'])+"\n")
		if 'SOG' in params: out.write("	SOG:                "+str(params['SOG'])+"\n")
		if 'PositionAccuracy' in params: out.write("	PositionAccuracy:   "+str(params['PositionAccuracy'])+"\n")
		if 'longitude' in params: out.write("	longitude:          "+str(params['longitude'])+"\n")
		if 'latitude' in params: out.write("	latitude:           "+str(params['latitude'])+"\n")
		if 'COG' in params: out.write("	COG:                "+str(params['COG'])+"\n")
		if 'TrueHeading' in params: out.write("	TrueHeading:        "+str(params['TrueHeading'])+"\n")
		if 'TimeStamp' in params: out.write("	TimeStamp:          "+str(params['TimeStamp'])+"\n")
		if 'Spare' in params: out.write("	Spare:              "+str(params['Spare'])+"\n")
		if 'cs_unit' in params: out.write("	cs_unit:            "+str(params['cs_unit'])+"\n")
		if 'display_flag' in params: out.write("	display_flag:       "+str(params['display_flag'])+"\n")
		if 'dsc_flag' in params: out.write("	dsc_flag:           "+str(params['dsc_flag'])+"\n")
		if 'band_flag' in params: out.write("	band_flag:          "+str(params['band_flag'])+"\n")
		if 'msg22_flag' in params: out.write("	msg22_flag:         "+str(params['msg22_flag'])+"\n")
		if 'mode_flag' in params: out.write("	mode_flag:          "+str(params['mode_flag'])+"\n")
		if 'RAIM' in params: out.write("	RAIM:               "+str(params['RAIM'])+"\n")
		if 'CommStateSelector' in params: out.write("	CommStateSelector:  "+str(params['CommStateSelector'])+"\n")
		if 'CommState' in params: out.write("	CommState:          "+str(params['CommState'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'sql'==format:
		sqlInsertStr(params,out,dbType=dbType)
	elif 'kml'==format:
		printKml(params,out)
	elif 'kml-full'==format:
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		out.write("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n")
		out.write("<Document>\n")
		out.write("	<name>positionb</name>\n")
		printKml(params,out)
		out.write("</Document>\n")
		out.write("</kml>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

RepeatIndicatorEncodeLut = {
	'default':'0',
	'do not repeat any more':'3',
	} #RepeatIndicatorEncodeLut

RepeatIndicatorDecodeLut = {
	'0':'default',
	'3':'do not repeat any more',
	} # RepeatIndicatorEncodeLut

SOGEncodeLut = {
	'102.2 knots or higher':'102.2',
	} #SOGEncodeLut

SOGDecodeLut = {
	'102.2':'102.2 knots or higher',
	} # SOGEncodeLut

PositionAccuracyEncodeLut = {
	'low (greater than 10 m)':'0',
	'high (less than 10 m)':'1',
	} #PositionAccuracyEncodeLut

PositionAccuracyDecodeLut = {
	'0':'low (greater than 10 m)',
	'1':'high (less than 10 m)',
	} # PositionAccuracyEncodeLut

TimeStampEncodeLut = {
	'not available/default':'60',
	'manual input':'61',
	'dead reckoning':'62',
	'inoperative':'63',
	} #TimeStampEncodeLut

TimeStampDecodeLut = {
	'60':'not available/default',
	'61':'manual input',
	'62':'dead reckoning',
	'63':'inoperative',
	} # TimeStampEncodeLut

cs_unitEncodeLut = {
	'Class B SOTDMA unit':'False',
	'Class B CS unit':'True',
	} #cs_unitEncodeLut

cs_unitDecodeLut = {
	'False':'Class B SOTDMA unit',
	'True':'Class B CS unit',
	} # cs_unitEncodeLut

display_flagEncodeLut = {
	'No display':'False',
	'Integrated Display':'True',
	} #display_flagEncodeLut

display_flagDecodeLut = {
	'False':'No display',
	'True':'Integrated Display',
	} # display_flagEncodeLut

dsc_flagEncodeLut = {
	'No DSC function':'False',
	'Has DSC':'True',
	} #dsc_flagEncodeLut

dsc_flagDecodeLut = {
	'False':'No DSC function',
	'True':'Has DSC',
	} # dsc_flagEncodeLut

band_flagEncodeLut = {
	'Upper 525 kHz band':'False',
	'Whole marine band':'True',
	} #band_flagEncodeLut

band_flagDecodeLut = {
	'False':'Upper 525 kHz band',
	'True':'Whole marine band',
	} # band_flagEncodeLut

msg22_flagEncodeLut = {
	'No freq management':'False',
	'Freq management with msg 22':'True',
	} #msg22_flagEncodeLut

msg22_flagDecodeLut = {
	'False':'No freq management',
	'True':'Freq management with msg 22',
	} # msg22_flagEncodeLut

mode_flagEncodeLut = {
	'Autonomous and continuous mode':'False',
	'Assigned mode':'True',
	} #mode_flagEncodeLut

mode_flagDecodeLut = {
	'False':'Autonomous and continuous mode',
	'True':'Assigned mode',
	} # mode_flagEncodeLut

RAIMEncodeLut = {
	'not in use':'False',
	'in use':'True',
	} #RAIMEncodeLut

RAIMDecodeLut = {
	'False':'not in use',
	'True':'in use',
	} # RAIMEncodeLut

CommStateSelectorEncodeLut = {
	'SOTDMA':'0',
	'ITDMA':'1',
	} #CommStateSelectorEncodeLut

CommStateSelectorDecodeLut = {
	'0':'SOTDMA',
	'1':'ITDMA',
	} # CommStateSelectorEncodeLut

######################################################################
# SQL SUPPORT
######################################################################

dbTableName='positionb'
'Database table name'

def sqlCreateStr(outfile=sys.stdout, fields=None, extraFields=None
		,addCoastGuardFields=True
		,dbType='postgres'
		):
	'''
	Return the SQL CREATE command for this message type
	@param outfile: file like object to print to.
	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@param dbType: Which flavor of database we are using so that the create is tailored ('sqlite' or 'postgres')
	@type addCoastGuardFields: bool
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	# FIX: should this sqlCreate be the same as in LaTeX (createFuncName) rather than hard coded?
	outfile.write(str(sqlCreate(fields,extraFields,addCoastGuardFields,dbType=dbType)))

def sqlCreate(fields=None, extraFields=None, addCoastGuardFields=True, dbType='postgres'):
	'''
	Return the sqlhelp object to create the table.

	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@param dbType: Which flavor of database we are using so that the create is tailored ('sqlite' or 'postgres')
	@return: An object that can be used to generate a return
	@rtype: sqlhelp.create
	'''
	if None == fields: fields = fieldList
	import sqlhelp
	c = sqlhelp.create('positionb',dbType=dbType)
	c.addPrimaryKey()
	if 'MessageID' in fields: c.addInt ('MessageID')
	if 'RepeatIndicator' in fields: c.addInt ('RepeatIndicator')
	if 'UserID' in fields: c.addInt ('UserID')
	if 'Reserved1' in fields: c.addInt ('Reserved1')
	if 'SOG' in fields: c.addDecimal('SOG',4,1)
	if 'PositionAccuracy' in fields: c.addInt ('PositionAccuracy')
	if dbType != 'postgres':
		if 'longitude' in fields: c.addDecimal('longitude',8,5)
	if dbType != 'postgres':
		if 'latitude' in fields: c.addDecimal('latitude',8,5)
	if 'COG' in fields: c.addDecimal('COG',4,1)
	if 'TrueHeading' in fields: c.addInt ('TrueHeading')
	if 'TimeStamp' in fields: c.addInt ('TimeStamp')
	if 'Spare' in fields: c.addInt ('Spare')
	if 'cs_unit' in fields: c.addBool('cs_unit')
	if 'display_flag' in fields: c.addBool('display_flag')
	if 'dsc_flag' in fields: c.addBool('dsc_flag')
	if 'band_flag' in fields: c.addBool('band_flag')
	if 'msg22_flag' in fields: c.addBool('msg22_flag')
	if 'mode_flag' in fields: c.addBool('mode_flag')
	if 'RAIM' in fields: c.addBool('RAIM')
	if 'CommStateSelector' in fields: c.addInt ('CommStateSelector')
	if 'CommState' in fields: c.addInt ('CommState')

	if addCoastGuardFields:
		# c.addInt('cg_s_rssi')     # Relative signal strength indicator
		# c.addInt('cg_d_strength')        # dBm receive strength
		# c.addVarChar('cg_x',10) # Idonno
		c.addInt('cg_t_arrival')        # Receive timestamp from the AIS equipment 'T'
		c.addInt('cg_s_slotnum')        # Slot received in
		c.addVarChar('cg_r',15)   # Receiver station ID  -  should usually be an MMSI, but sometimes is a string
		c.addInt('cg_sec')        # UTC seconds since the epoch

		c.addTimestamp('cg_timestamp') # UTC decoded cg_sec - not actually in the data stream

	if dbType == 'postgres':
		#--- EPSG 4326 : WGS 84
		#INSERT INTO "spatial_ref_sys" ("srid","auth_name","auth_srid","srtext","proj4text") VALUES (4326,'EPSG',4326,'GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0,AUTHORITY["EPSG","8901"]],UNIT["degree",0.01745329251994328,AUTHORITY["EPSG","9122"]],AUTHORITY["EPSG","4326"]]','+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs ');
		c.addPostGIS('Position','POINT',2,SRID=4326);

	return c

def sqlInsertStr(params, outfile=sys.stdout, extraParams=None, dbType='postgres'):
	'''
	Return the SQL INSERT command for this message type
	@param params: dictionary of values keyed by field name
	@param outfile: file like object to print to.
	@param extraParams: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlInsert(params,extraParams,dbType=dbType)))


def sqlInsert(params,extraParams=None,dbType='postgres'):
	'''
	Give the SQL INSERT statement
	@param params: dict keyed by field name of values
	@param extraParams: any extra fields that you have created beyond the normal ais message fields
	@rtype: sqlhelp.insert
	@return: insert class instance
	@todo: allow optional type checking of params?
	@warning: this will take invalid keys happily and do what???
	'''
	import sqlhelp
	i = sqlhelp.insert('positionb',dbType=dbType)

	if dbType=='postgres':
		finished = []
		for key in params:
			if key in finished: 
				continue

			if key not in toPgFields and key not in fromPgFields:
				if type(params[key])==Decimal: i.add(key,float(params[key]))
				else: i.add(key,params[key])
			else:
				if key in fromPgFields:
					val = params[key]
				        # Had better be a WKT type like POINT(-88.1 30.321)
					i.addPostGIS(key,val)
					finished.append(key)
				else:
					# Need to construct the type.
					pgName = toPgFields[key]
					#valStr='GeomFromText(\''+pgTypes[pgName]+'('
					valStr=pgTypes[pgName]+'('
					vals = []
					for nonPgKey in fromPgFields[pgName]:
						vals.append(str(params[nonPgKey]))
						finished.append(nonPgKey)
					valStr+=' '.join(vals)+')'
					i.addPostGIS(pgName,valStr)
	else:
		for key in params: 
			if type(params[key])==Decimal: i.add(key,float(params[key]))
			else: i.add(key,params[key])

	if None != extraParams:
		for key in extraParams: 
			i.add(key,extraParams[key])

	return i

######################################################################
# LATEX SUPPORT
######################################################################

def latexDefinitionTable(outfile=sys.stdout
		):
	'''
	Return the LaTeX definition table for this message type
	@param outfile: file like object to print to.
	@type outfile: file obj
	@return: LaTeX table string via the outfile
	@rtype: str

	'''
	o = outfile

	o.write('''
\\begin{table}%[htb]
\\centering
\\begin{tabular}{|l|c|l|}
\\hline
Parameter & Number of bits & Description 
\\\\  \\hline\\hline
MessageID & 6 & AIS message number.  Must be 18 \\\\ \hline 
RepeatIndicator & 2 & Indicated how many times a message has been repeated \\\\ \hline 
UserID & 30 & Unique ship identification number (MMSI) \\\\ \hline 
Reserved1 & 8 & Reseverd for definition by a compentent regional or local authority.  Should be set to zero. \\\\ \hline 
SOG & 10 & Speed over ground \\\\ \hline 
PositionAccuracy & 1 & Accuracy of positioning fixes \\\\ \hline 
longitude & 28 & Location of the vessel  East West location \\\\ \hline 
latitude & 27 & Location of the vessel  North South location \\\\ \hline 
COG & 12 & Course over ground \\\\ \hline 
TrueHeading & 9 & True heading (relative to true North) \\\\ \hline 
TimeStamp & 6 & UTC second when the report was generated \\\\ \hline 
Spare & 2 & Not used.  Should be set to zero. \\\\ \hline 
cs\_unit & 1 & Does this unit do Carrier Sense? \\\\ \hline 
display\_flag & 1 & Does this class B unit have an integrated display? \\\\ \hline 
dsc\_flag & 1 & Does it have dedicated or time-shared DSC radio function? \\\\ \hline 
band\_flag & 1 & How flexible is the freq handling of the unit? \\\\ \hline 
msg22\_flag & 1 & Can the unit handle msg 22? \\\\ \hline 
mode\_flag & 1 & Assigned mode wrt to VDL slots \\\\ \hline 
RAIM & 1 & Receiver autonomous integrity monitoring flag \\\\ \hline 
CommStateSelector & 1 & SOTDMA or ITDMA \\\\ \hline 
CommState & 19 & Not decoded by this software yet\\\\ \\hline \\hline
Total bits & 168 & Appears to take 1 slot \\\\ \\hline
\\end{tabular}
\\caption{AIS message number 18: Standard Class B equipment position report}
\\label{tab:positionb}
\\end{table}
''')

######################################################################
# Text Definition
######################################################################

def textDefinitionTable(outfile=sys.stdout
		,delim='\t'
		):
	'''
	Return the text definition table for this message type
	@param outfile: file like object to print to.
	@type outfile: file obj
	@return: text table string via the outfile
	@rtype: str

	'''
	o = outfile
	o.write('''Parameter'''+delim+'Number of bits'''+delim+'''Description 
MessageID'''+delim+'''6'''+delim+'''AIS message number.  Must be 18
RepeatIndicator'''+delim+'''2'''+delim+'''Indicated how many times a message has been repeated
UserID'''+delim+'''30'''+delim+'''Unique ship identification number (MMSI)
Reserved1'''+delim+'''8'''+delim+'''Reseverd for definition by a compentent regional or local authority.  Should be set to zero.
SOG'''+delim+'''10'''+delim+'''Speed over ground
PositionAccuracy'''+delim+'''1'''+delim+'''Accuracy of positioning fixes
longitude'''+delim+'''28'''+delim+'''Location of the vessel  East West location
latitude'''+delim+'''27'''+delim+'''Location of the vessel  North South location
COG'''+delim+'''12'''+delim+'''Course over ground
TrueHeading'''+delim+'''9'''+delim+'''True heading (relative to true North)
TimeStamp'''+delim+'''6'''+delim+'''UTC second when the report was generated
Spare'''+delim+'''2'''+delim+'''Not used.  Should be set to zero.
cs_unit'''+delim+'''1'''+delim+'''Does this unit do Carrier Sense?
display_flag'''+delim+'''1'''+delim+'''Does this class B unit have an integrated display?
dsc_flag'''+delim+'''1'''+delim+'''Does it have dedicated or time-shared DSC radio function?
band_flag'''+delim+'''1'''+delim+'''How flexible is the freq handling of the unit?
msg22_flag'''+delim+'''1'''+delim+'''Can the unit handle msg 22?
mode_flag'''+delim+'''1'''+delim+'''Assigned mode wrt to VDL slots
RAIM'''+delim+'''1'''+delim+'''Receiver autonomous integrity monitoring flag
CommStateSelector'''+delim+'''1'''+delim+'''SOTDMA or ITDMA
CommState'''+delim+'''19'''+delim+'''Not decoded by this software yet
Total bits'''+delim+'''168'''+delim+'''Appears to take 1 slot''')


######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['MessageID'] = 18
	params['RepeatIndicator'] = 1
	params['UserID'] = 1193046
	params['Reserved1'] = 0
	params['SOG'] = Decimal('101.9')
	params['PositionAccuracy'] = 1
	params['longitude'] = Decimal('-122.16328055555556')
	params['latitude'] = Decimal('37.424458333333334')
	params['COG'] = Decimal('34.5')
	params['TrueHeading'] = 41
	params['TimeStamp'] = 35
	params['Spare'] = 0
	params['cs_unit'] = False
	params['display_flag'] = False
	params['dsc_flag'] = False
	params['band_flag'] = False
	params['msg22_flag'] = False
	params['mode_flag'] = False
	params['RAIM'] = False
	params['CommStateSelector'] = 0
	params['CommState'] = 0

	return params

class Testpositionb(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the positionb message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['RepeatIndicator'],params['RepeatIndicator'])
		self.failUnlessEqual(r['UserID'],params['UserID'])
		self.failUnlessEqual(r['Reserved1'],params['Reserved1'])
		self.failUnlessAlmostEqual(r['SOG'],params['SOG'],1)
		self.failUnlessEqual(r['PositionAccuracy'],params['PositionAccuracy'])
		self.failUnlessAlmostEqual(r['longitude'],params['longitude'],5)
		self.failUnlessAlmostEqual(r['latitude'],params['latitude'],5)
		self.failUnlessAlmostEqual(r['COG'],params['COG'],1)
		self.failUnlessEqual(r['TrueHeading'],params['TrueHeading'])
		self.failUnlessEqual(r['TimeStamp'],params['TimeStamp'])
		self.failUnlessEqual(r['Spare'],params['Spare'])
		self.failUnlessEqual(r['cs_unit'],params['cs_unit'])
		self.failUnlessEqual(r['display_flag'],params['display_flag'])
		self.failUnlessEqual(r['dsc_flag'],params['dsc_flag'])
		self.failUnlessEqual(r['band_flag'],params['band_flag'])
		self.failUnlessEqual(r['msg22_flag'],params['msg22_flag'])
		self.failUnlessEqual(r['mode_flag'],params['mode_flag'])
		self.failUnlessEqual(r['RAIM'],params['RAIM'])
		self.failUnlessEqual(r['CommStateSelector'],params['CommStateSelector'])
		self.failUnlessEqual(r['CommState'],params['CommState'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "positionb" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "positionb" AIS message')
	parser.add_option('--RepeatIndicator-field', dest='RepeatIndicatorField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--UserID-field', dest='UserIDField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--SOG-field', dest='SOGField',default=Decimal('102.3'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--PositionAccuracy-field', dest='PositionAccuracyField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--longitude-field', dest='longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--latitude-field', dest='latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--COG-field', dest='COGField',default=Decimal('360'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--TrueHeading-field', dest='TrueHeadingField',default=511,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--TimeStamp-field', dest='TimeStampField',default=60,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--cs_unit-field', dest='cs_unitField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--display_flag-field', dest='display_flagField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dsc_flag-field', dest='dsc_flagField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--band_flag-field', dest='band_flagField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--msg22_flag-field', dest='msg22_flagField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--mode_flag-field', dest='mode_flagField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--RAIM-field', dest='RAIMField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--CommStateSelector-field', dest='CommStateSelectorField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--CommState-field', dest='CommStateField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

def main():
	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to write for encoding ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv','sql' , 'kml','kml-full')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	parser.add_option('-c','--sql-create',dest='sqlCreate',default=False,action='store_true',
			  help='Print out an sql create command for the table.')

	parser.add_option('--latex-table',dest='latexDefinitionTable',default=False,action='store_true',
			  help='Print a LaTeX table of the type')

	parser.add_option('--text-table',dest='textDefinitionTable',default=False,action='store_true',
			  help='Print delimited table of the type (for Word table importing)')
	parser.add_option('--delimt-text-table',dest='delimTextDefinitionTable',default='\t'
			  ,help='Delimiter for text table [default: \'%default\'](for Word table importing)')


	dbChoices = ('sqlite','postgres')
	parser.add_option('-D','--db-type',dest='dbType',default='postgres'
			  ,choices=dbChoices,type='choice'
			  ,help='What kind of database ('+', '.join(dbChoices)+') [default: %default]')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.RepeatIndicatorField: parser.error("missing value for RepeatIndicatorField")
		if None==options.UserIDField: parser.error("missing value for UserIDField")
		if None==options.SOGField: parser.error("missing value for SOGField")
		if None==options.PositionAccuracyField: parser.error("missing value for PositionAccuracyField")
		if None==options.longitudeField: parser.error("missing value for longitudeField")
		if None==options.latitudeField: parser.error("missing value for latitudeField")
		if None==options.COGField: parser.error("missing value for COGField")
		if None==options.TrueHeadingField: parser.error("missing value for TrueHeadingField")
		if None==options.TimeStampField: parser.error("missing value for TimeStampField")
		if None==options.cs_unitField: parser.error("missing value for cs_unitField")
		if None==options.display_flagField: parser.error("missing value for display_flagField")
		if None==options.dsc_flagField: parser.error("missing value for dsc_flagField")
		if None==options.band_flagField: parser.error("missing value for band_flagField")
		if None==options.msg22_flagField: parser.error("missing value for msg22_flagField")
		if None==options.mode_flagField: parser.error("missing value for mode_flagField")
		if None==options.RAIMField: parser.error("missing value for RAIMField")
		if None==options.CommStateSelectorField: parser.error("missing value for CommStateSelectorField")
		if None==options.CommStateField: parser.error("missing value for CommStateField")
		msgDict={
			'MessageID': '18',
			'RepeatIndicator': options.RepeatIndicatorField,
			'UserID': options.UserIDField,
			'Reserved1': '0',
			'SOG': options.SOGField,
			'PositionAccuracy': options.PositionAccuracyField,
			'longitude': options.longitudeField,
			'latitude': options.latitudeField,
			'COG': options.COGField,
			'TrueHeading': options.TrueHeadingField,
			'TimeStamp': options.TimeStampField,
			'Spare': '0',
			'cs_unit': options.cs_unitField,
			'display_flag': options.display_flagField,
			'dsc_flag': options.dsc_flagField,
			'band_flag': options.band_flagField,
			'msg22_flag': options.msg22_flagField,
			'mode_flag': options.mode_flagField,
			'RAIM': options.RAIMField,
			'CommStateSelector': options.CommStateSelectorField,
			'CommState': options.CommStateField,
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		    # FIX: figure out if this might be necessary at compile time
		    #print "bitLen",len(bits)
		    bitLen=len(bits)
		    if bitLen%6!=0:
			bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
		    #print "result:",binary.bitvectoais6(bits)[0]
		    print binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: 
		    #bitLen=len(bits)
                    #if bitLen%6!=0:
		    #	bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
                    import aisutils.uscg as uscg
                    nmea = uscg.create_nmea(bits)
                    print nmea
                    #
                    #


                    #sys.exit("FIX: need to implement creating nmea capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')


	if options.sqlCreate:
		sqlCreateStr(outfile,options.fieldList,dbType=options.dbType)

	if options.latexDefinitionTable:
		latexDefinitionTable(outfile)

	# For conversion to word tables
	if options.textDefinitionTable:
		textDefinitionTable(outfile,options.delimTextDefinitionTable)

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		if len(args)==0: args = sys.stdin
		for msg in args:
			bv = None

			if msg[0] in ('$','!') and msg[3:6] in ('VDM','VDO'):
				# Found nmea
				# FIX: do checksum
				bv = binary.ais6tobitvec(msg.split(',')[5])
			else: # either binary or nmeapayload... expect mostly nmeapayloads
				# assumes that an all 0 and 1 string can not be a nmeapayload
				binaryMsg=True
				for c in msg:
					if c not in ('0','1'):
						binaryMsg=False
						break
				if binaryMsg:
					bv = BitVector(bitstring=msg)
				else: # nmeapayload
					bv = binary.ais6tobitvec(msg)

			printFields(decode(bv)
				    ,out=outfile
				    ,format=options.outputType
				    ,fieldList=options.fieldList
				    ,dbType=options.dbType
				    )

############################################################
if __name__=='__main__':
    main()
